BalancedBaggingClassifier — Version 0.11.0.dev0

# BalancedBaggingClassifier

class imblearn.ensemble.BalancedBaggingClassifier(estimator=None, n_estimators=10, *, max_samples=1.0, max_features=1.0, bootstrap=True, bootstrap_features=False, oob_score=False, warm_start=False, sampling_strategy='auto', replacement=False, n_jobs=None, random_state=None, verbose=0, sampler=None, base_estimator='deprecated')

  A Bagging classifier with additional balancing.

  This implementation of Bagging is similar to the scikit-learn implementation. It includes an additional step to balance the training set at fit time using a given sampler.

  This classifier can serves as a basis to implement various methods such as Exactly Balanced Bagging [6], Roughly Balanced Bagging [7], Over-Bagging [6], or SMOTE-Bagging [8].

  Read more in the User Guide.

  Parameters
    estimatorestimator object, default=None

      The base estimator to fit on random subsets of the dataset. If None, then the base estimator is a decision tree.

      New in version 0.10.

    n_estimatorsint, default=10

      The number of base estimators in the ensemble.

    max_samplesint or float, default=1.0

      The number of samples to draw from X to train each base estimator.

      - If int, then draw `max_samples` samples.

      - If float, then draw `max_samples * X.shape[0]` samples.

    max_featuresint or float, default=1.0

      The number of features to draw from X to train each base estimator.

      - If int, then draw `max_features` features.

      - If float, then draw `max_features * X.shape[1]` features.

    bootstrapbool, default=True

      Whether samples are drawn with replacement.

      Note

      Note that this bootstrap will be generated from the resampled dataset.

    bootstrap_featuresbool, default=False

      Whether features are drawn with replacement.

    oob_scorebool, default=False

      Whether to use out-of-bag samples to estimate the generalization error.

    warm_startbool, default=False

      When set to True, reuse the solution of the previous call to fit and add more estimators to the ensemble, otherwise, just fit a whole new ensemble.

    sampling_strategyfloat, str, dict, callable, default=’auto’

      Sampling information to sample the data set.

      - When `float`, it corresponds to the desired ratio of the number of samples in the minority class over the number of samples in the majority class after resampling. Therefore, the ratio is expressed as \(\alpha_{us} = N_{m} / N_{rM}\) where \(N_{m}\) is the number of samples in the minority class and \(N_{rM}\) is the number of samples in the majority class after resampling.

        Warning

        `float` is only available for binary classification. An error is raised for multi-class classification.

      - When `str`, specify the class targeted by the resampling. The number of samples in the different classes will be equalized. Possible choices are:

        > `'majority'`: resample only the majority class;

        > `'not minority'`: resample all classes but the minority class;

        > `'not majority'`: resample all classes but the majority class;

        > `'all'`: resample all classes;

        > `'auto'`: equivalent to `'not minority'`.

      - When `dict`, the keys correspond to the targeted classes. The values correspond to the desired number of samples for each targeted class.

      - When callable, function taking `y` and returns a `dict`. The keys correspond to the targeted classes. The values correspond to the desired number of samples for each class.

    replacementbool, default=False

      Whether or not to randomly sample with replacement or not when `sampler is None`, corresponding to a `RandomUnderSampler`.

    n_jobsint, default=None

      Number of CPU cores used during the cross-validation loop. `None` means 1 unless in a `joblib.parallel_backend` context. `-1` means using all processors. See Glossary for more details.

    random_stateint, RandomState instance, default=None

      Control the randomization of the algorithm.

      - If int, `random_state` is the seed used by the random number generator;

      - If `RandomState` instance, random_state is the random number generator;

      - If `None`, the random number generator is the `RandomState` instance used by `np.random`.

    verboseint, default=0

      Controls the verbosity of the building process.

    samplersampler object, default=None

      The sampler used to balanced the dataset before to bootstrap (if `bootstrap=True`) and `fit` a base estimator. By default, a `RandomUnderSampler` is used.

      New in version 0.8.

    base_estimatorestimator object, default=None

      The base estimator to fit on random subsets of the dataset. If None, then the base estimator is a decision tree.

      Deprecated since version 0.10: `base_estimator` was renamed to `estimator` in version 0.10 and will be removed in 0.12.

  Attributes
    `estimator_` estimator

      Estimator used to grow the ensemble.

    `base_estimator_` estimator

      Estimator used to grow the ensemble.

    `n_features_` int

      Number of features when `fit` is performed.

    estimators_list of estimators

      The collection of fitted base estimators.

    sampler_sampler object

      The validate sampler created from the `sampler` parameter.

    `estimators_samples_` list of ndarray

      The subset of drawn samples for each base estimator.

    estimators_features_list of ndarray

      The subset of drawn features for each base estimator.

    classes_ndarray of shape (n_classes,)

      The classes labels.

    n_classes_int or list

      The number of classes.

    oob_score_float

      Score of the training dataset obtained using an out-of-bag estimate.

    oob_decision_function_ndarray of shape (n_samples, n_classes)

      Decision function computed with out-of-bag estimate on the training set. If n_estimators is small it might be possible that a data point was never left out during the bootstrap. In this case, `oob_decision_function_` might contain NaN.

    n_features_in_int

      Number of features in the input dataset.

      New in version 0.9.

    feature_names_in_ndarray of shape (`n_features_in_`,)

      Names of features seen during `fit`. Defined only when `X` has feature names that are all strings.

      New in version 0.9.

  See also

  `BalancedRandomForestClassifier`

    Random forest applying random-under sampling to balance the different bootstraps.

  `EasyEnsembleClassifier`

    Ensemble of AdaBoost classifier trained on balanced bootstraps.

  `RUSBoostClassifier`

    AdaBoost classifier were each bootstrap is balanced using random-under sampling at each round of boosting.

  Notes

  This is possible to turn this classifier into a balanced random forest [5] by passing a `DecisionTreeClassifier` with `max_features='auto'` as a base estimator.

  See Compare ensemble classifiers using resampling.

  References

  1

    L. Breiman, “Pasting small votes for classification in large databases and on-line”, Machine Learning, 36(1), 85-103, 1999.

  2

    L. Breiman, “Bagging predictors”, Machine Learning, 24(2), 123-140, 1996.

  3

    T. Ho, “The random subspace method for constructing decision forests”, Pattern Analysis and Machine Intelligence, 20(8), 832-844, 1998.

  4

    G. Louppe and P. Geurts, “Ensembles on Random Patches”, Machine Learning and Knowledge Discovery in Databases, 346-361, 2012.

  5

    C. Chen Chao, A. Liaw, and L. Breiman. “Using random forest to learn imbalanced data.” University of California, Berkeley 110, 2004.

  6(1,2)

    R. Maclin, and D. Opitz. “An empirical evaluation of bagging and boosting.” AAAI/IAAI 1997 (1997): 546-551.

  7

    S. Hido, H. Kashima, and Y. Takahashi. “Roughly balanced bagging for imbalanced data.” Statistical Analysis and Data Mining: The ASA Data Science Journal 2.5‐6 (2009): 412-426.

  8

    S. Wang, and X. Yao. “Diversity analysis on imbalanced data sets by using ensemble models.” 2009 IEEE symposium on computational intelligence and data mining. IEEE, 2009.

  Examples

      >>> from collections import Counter
      >>> from sklearn.datasets import make_classification
      >>> from sklearn.model_selection import train_test_split
      >>> from sklearn.metrics import confusion_matrix
      >>> from imblearn.ensemble import BalancedBaggingClassifier
      >>> X, y = make_classification(n_classes=2, class_sep=2,
      ... weights=[0.1, 0.9], n_informative=3, n_redundant=1, flip_y=0,
      ... n_features=20, n_clusters_per_class=1, n_samples=1000, random_state=10)
      >>> print('Original dataset shape %s' % Counter(y))
      Original dataset shape Counter({1: 900, 0: 100})
      >>> X_train, X_test, y_train, y_test = train_test_split(X, y,
      ...                                                     random_state=0)
      >>> bbc = BalancedBaggingClassifier(random_state=42)
      >>> bbc.fit(X_train, y_train)
      BalancedBaggingClassifier(...)
      >>> y_pred = bbc.predict(X_test)
      >>> print(confusion_matrix(y_test, y_pred))
      [[ 23   0]
       [  2 225]]

  Methods

  `decision_function`(X) | Average of the decision functions of the base classifiers.
  `fit`(X, y) | Build a Bagging ensemble of estimators from the training set (X, y).
  `get_metadata_routing`() | Get metadata routing of this object.
  `get_params`([deep]) | Get parameters for this estimator.
  `predict`(X) | Predict class for X.
  `predict_log_proba`(X) | Predict class log-probabilities for X.
  `predict_proba`(X) | Predict class probabilities for X.
  `score`(X, y[, sample_weight]) | Return the mean accuracy on the given test data and labels.
  `set_fit_request`(*[, sample_weight]) | Request metadata passed to the `fit` method.
  `set_params`(**params) | Set the parameters of this estimator.
  `set_score_request`(*[, sample_weight]) | Request metadata passed to the `score` method.

  property base_estimator_

    Estimator used to grow the ensemble.

  decision_function(X)

    Average of the decision functions of the base classifiers.

    Parameters
      X{array-like, sparse matrix} of shape (n_samples, n_features)

        The training input samples. Sparse matrices are accepted only if they are supported by the base estimator.

    Returns
      scorendarray of shape (n_samples, k)

        The decision function of the input samples. The columns correspond to the classes in sorted order, as they appear in the attribute `classes_`. Regression and binary classification are special cases with `k == 1`, otherwise `k==n_classes`.

  property estimator_

    Estimator used to grow the ensemble.

  property estimators_samples_

    The subset of drawn samples for each base estimator.

    Returns a dynamically generated list of indices identifying the samples used for fitting each member of the ensemble, i.e., the in-bag samples.

    Note: the list is re-created at each call to the property in order to reduce the object memory footprint by not storing the sampling data. Thus fetching the property may be slower than expected.

  fit(X, y)

    Build a Bagging ensemble of estimators from the training set (X, y).

    Parameters
      X{array-like, sparse matrix} of shape (n_samples, n_features)

        The training input samples. Sparse matrices are accepted only if they are supported by the base estimator.

      yarray-like of shape (n_samples,)

        The target values (class labels in classification, real numbers in regression).

    Returns
      selfobject

        Fitted estimator.

  get_metadata_routing()

    Get metadata routing of this object.

    Please check User Guide on how the routing mechanism works.

    Returns
      routingMetadataRequest

        A `MetadataRequest` encapsulating routing information.

  get_params(deep=True)

    Get parameters for this estimator.

    Parameters
      deepbool, default=True

        If True, will return the parameters for this estimator and contained subobjects that are estimators.

    Returns
      paramsdict

        Parameter names mapped to their values.

  property n_features_

    Number of features when `fit` is performed.

  predict(X)

    Predict class for X.

    The predicted class of an input sample is computed as the class with the highest mean predicted probability. If base estimators do not implement a `predict_proba` method, then it resorts to voting.

    Parameters
      X{array-like, sparse matrix} of shape (n_samples, n_features)

        The training input samples. Sparse matrices are accepted only if they are supported by the base estimator.

    Returns
      yndarray of shape (n_samples,)

        The predicted classes.

  predict_log_proba(X)

    Predict class log-probabilities for X.

    The predicted class log-probabilities of an input sample is computed as the log of the mean predicted class probabilities of the base estimators in the ensemble.

    Parameters
      X{array-like, sparse matrix} of shape (n_samples, n_features)

        The training input samples. Sparse matrices are accepted only if they are supported by the base estimator.

    Returns
      pndarray of shape (n_samples, n_classes)

        The class log-probabilities of the input samples. The order of the classes corresponds to that in the attribute classes_.

  predict_proba(X)

    Predict class probabilities for X.

    The predicted class probabilities of an input sample is computed as the mean predicted class probabilities of the base estimators in the ensemble. If base estimators do not implement a `predict_proba` method, then it resorts to voting and the predicted class probabilities of an input sample represents the proportion of estimators predicting each class.

    Parameters
      X{array-like, sparse matrix} of shape (n_samples, n_features)

        The training input samples. Sparse matrices are accepted only if they are supported by the base estimator.

    Returns
      pndarray of shape (n_samples, n_classes)

        The class probabilities of the input samples. The order of the classes corresponds to that in the attribute classes_.

  score(X, y, sample_weight=None)

    Return the mean accuracy on the given test data and labels.

    In multi-label classification, this is the subset accuracy which is a harsh metric since you require for each sample that each label set be correctly predicted.

    Parameters
      Xarray-like of shape (n_samples, n_features)

        Test samples.

      yarray-like of shape (n_samples,) or (n_samples, n_outputs)

        True labels for `X`.

      sample_weightarray-like of shape (n_samples,), default=None

        Sample weights.

    Returns
      scorefloat

        Mean accuracy of `self.predict(X)` w.r.t. `y`.

  set_fit_request(*, sample_weight: Union[bool, None, str] = '$UNCHANGED$') → imblearn.ensemble._bagging.BalancedBaggingClassifier

    Request metadata passed to the `fit` method.

    Note that this method is only relevant if `enable_metadata_routing=True` (see `sklearn.set_config`). Please see User Guide on how the routing mechanism works.

    The options for each parameter are:

    - `True`: metadata is requested, and passed to `fit` if provided. The request is ignored if metadata is not provided.

    - `False`: metadata is not requested and the meta-estimator will not pass it to `fit`.

    - `None`: metadata is not requested, and the meta-estimator will raise an error if the user provides it.

    - `str`: metadata should be passed to the meta-estimator with this given alias instead of the original name.

    The default (`sklearn.utils.metadata_routing.UNCHANGED`) retains the existing request. This allows you to change the request for some parameters and not others.

    New in version 1.3.

    Note

    This method is only relevant if this estimator is used as a sub-estimator of a meta-estimator, e.g. used inside a `pipeline.Pipeline`. Otherwise it has no effect.

    Parameters
      sample_weightstr, True, False, or None, default=sklearn.utils.metadata_routing.UNCHANGED

        Metadata routing for `sample_weight` parameter in `fit`.

    Returns
      selfobject

        The updated object.

  set_params(**params)

    Set the parameters of this estimator.

    The method works on simple estimators as well as on nested objects (such as `Pipeline`). The latter have parameters of the form `<component>__<parameter>` so that it’s possible to update each component of a nested object.

    Parameters
      **paramsdict

        Estimator parameters.

    Returns
      selfestimator instance

        Estimator instance.

  set_score_request(*, sample_weight: Union[bool, None, str] = '$UNCHANGED$') → imblearn.ensemble._bagging.BalancedBaggingClassifier

    Request metadata passed to the `score` method.

    Note that this method is only relevant if `enable_metadata_routing=True` (see `sklearn.set_config`). Please see User Guide on how the routing mechanism works.

    The options for each parameter are:

    - `True`: metadata is requested, and passed to `score` if provided. The request is ignored if metadata is not provided.

    - `False`: metadata is not requested and the meta-estimator will not pass it to `score`.

    - `None`: metadata is not requested, and the meta-estimator will raise an error if the user provides it.

    - `str`: metadata should be passed to the meta-estimator with this given alias instead of the original name.

    The default (`sklearn.utils.metadata_routing.UNCHANGED`) retains the existing request. This allows you to change the request for some parameters and not others.

    New in version 1.3.

    Note

    This method is only relevant if this estimator is used as a sub-estimator of a meta-estimator, e.g. used inside a `pipeline.Pipeline`. Otherwise it has no effect.

    Parameters
      sample_weightstr, True, False, or None, default=sklearn.utils.metadata_routing.UNCHANGED

        Metadata routing for `sample_weight` parameter in `score`.

    Returns
      selfobject

        The updated object.

## Examples using `imblearn.ensemble.BalancedBaggingClassifier`

Fitting model on imbalanced datasets and how to fight bias

Fitting model on imbalanced datasets and how to fight bias

Bagging classifiers using sampler

Bagging classifiers using sampler

Compare ensemble classifiers using resampling

Compare ensemble classifiers using resampling

previous

RUSBoostClassifier

next

BalancedRandomForestClassifier